# sage_setup: distribution = sagemath-environment
r"""
Check for symengine_py
"""

# ****************************************************************************
#       Copyright (C) 2023 Dima Pasechnik
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#                  https://www.gnu.org/licenses/
# ****************************************************************************

from . import PythonModule
from .join_feature import JoinFeature


class symengine_py(JoinFeature):
    r"""
    A :class:`sage.features.Feature` describing the presence of the
    Python package :ref:`symengine_py <spkg_symengine_py>`.

    EXAMPLES::

        sage: from sage.features.symengine_py import symengine_py
        sage: symengine_py().is_present()                    # optional - symengine_py
        FeatureTestResult('symengine_py', True)
    """
    def __init__(self):
        r"""
        TESTS::

            sage: from sage.features.symengine_py import symengine_py
            sage: isinstance(symengine_py(), symengine_py)
            True
        """
        JoinFeature.__init__(self, 'symengine_py',
                             [PythonModule('symengine', spkg="symengine_py",
                                            url="https://pypi.org/project/symengine")])

def all_features():
    return [symengine_py()]



# sage_setup: distribution = sagemath-environment
r"""
Feature for testing the presence of SAT solvers
"""

from . import Executable, PythonModule


class Glucose(Executable):
    r"""
    A :class:`~sage.features.Feature` describing the presence of an
    executable from the :ref:`Glucose SAT solver <spkg_glucose>`.

    EXAMPLES::

        sage: from sage.features.sat import Glucose
        sage: GlucoseExecutable().is_present()                  # optional - glucose
        FeatureTestResult('glucose', True)
    """
    def __init__(self, executable="glucose"):
        r"""
        TESTS::

            sage: from sage.features.sat import Glucose
            sage: isinstance(Glucose(), Glucose)
            True
        """
        Executable.__init__(self, name=executable, executable=executable,
                            spkg="glucose", type="optional")


class Kissat(Executable):
    r"""
    A :class:`~sage.features.Feature` describing the presence of the
    :ref:`Kissat SAT solver <spkg_kissat>`.

    EXAMPLES::

        sage: from sage.features.sat import Kissat
        sage: Kissat().is_present()                             # optional - kissat
        FeatureTestResult('kissat', True)
    """
    def __init__(self):
        r"""
        TESTS::

            sage: from sage.features.sat import Kissat
            sage: isinstance(Kissat(), Kissat)
            True
        """
        Executable.__init__(self, name="kissat", executable="kissat",
                            spkg="kissat", type="optional")


class Pycosat(PythonModule):
    r"""
    A :class:`~sage.features.Feature` describing the presence of :ref:`spkg_pycosat`.

    EXAMPLES::

        sage: from sage.features.sat import Pycosat
        sage: PycosatExecutable().is_present()                  # optional - pycosat
        FeatureTestResult('pycosat', True)
    """
    def __init__(self):
        r"""
        TESTS::

            sage: from sage.features.sat import Pycosat
            sage: isinstance(Pycosat(), Pycosat)
            True
        """
        PythonModule.__init__(self, "pycosat",
                              spkg="pycosat", type="optional")


class Pycryptosat(PythonModule):
    r"""
    A :class:`~sage.features.Feature` describing the presence of :ref:`spkg_pycryptosat`.

    EXAMPLES::

        sage: from sage.features.sat import Pycryptosat
        sage: PycryptosatExecutable().is_present()              # optional - pycryptosat
        FeatureTestResult('pycryptosat', True)
    """
    def __init__(self):
        r"""
        TESTS::

            sage: from sage.features.sat import Pycryptosat
            sage: isinstance(Pycryptosat(), Pycryptosat)
            True
        """
        PythonModule.__init__(self, "pycryptosat",
                              spkg="pycryptosat", type="optional")


def all_features():
    return [Glucose(),
            Kissat(),
            Pycosat(),
            Pycryptosat()]



# sage_setup: distribution = sagemath-environment
r"""
Feature for testing the presence of ``palp``
"""
# ****************************************************************************
#       Copyright (C) 2022 Matthias Koeppe
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#                  https://www.gnu.org/licenses/
# ****************************************************************************

from . import Executable
from .join_feature import JoinFeature


class PalpExecutable(Executable):
    r"""
    A :class:`~sage.features.Feature` describing the presence of a :ref:`PALP <spkg_palp>` executable.

    INPUT:

    - ``palpprog`` -- string, one of ``"poly"``, ``"class"``, ``"nef"``, ``"cws"``.

    - ``suff`` -- string or ``None``.
    """
    def __init__(self, palpprog, suff=None):
        r"""
        TESTS::

            sage: from sage.features.palp import PalpExecutable
            sage: isinstance(PalpExecutable("poly", 5), PalpExecutable)
            True
        """
        if suff:
            Executable.__init__(self, f"palp_{palpprog}_{suff}d",
                                executable=f"{palpprog}-{suff}d.x",
                                spkg="palp", type="standard")
        else:
            Executable.__init__(self, f"palp_{palpprog}",
                                executable=f"{palpprog}.x",
                                spkg="palp", type="standard")

class Palp(JoinFeature):
    r"""
    A :class:`~sage.features.Feature` describing the presence of :ref:`PALP <spkg_palp>`.
    """
    def __init__(self):
        r"""
        TESTS::

            sage: from sage.features.palp import Palp
            sage: isinstance(Palp(), Palp)
            True
        """
        JoinFeature.__init__(self, "palp",
                             [PalpExecutable(palpprog, suff)
                              for palpprog in ("poly", "class", "nef", "cws")
                              for suff in (None, 4, 5, 6, 11)],
                             description="PALP")

def all_features():
    return [Palp()]



# sage_setup: distribution = sagemath-environment
r"""
Features for testing the presence of topcom executables
"""

# *****************************************************************************
#       Copyright (C) 2022-2024 Matthias Koeppe
#
#  Distributed under the terms of the GNU General Public License (GPL)
#  as published by the Free Software Foundation; either version 2 of
#  the License, or (at your option) any later version.
#                  https://www.gnu.org/licenses/
# *****************************************************************************

from . import Executable
from .join_feature import JoinFeature


class TOPCOMExecutable(Executable):
    r"""
    A :class:`~sage.features.Feature` which checks for executables from the :ref:`TOPCOM <spkg_topcom>` package.

    EXAMPLES::

        sage: from sage.features.topcom import TOPCOMExecutable
        sage: TOPCOMExecutable('points2allfinetriangs').is_present()    # optional - topcom
        FeatureTestResult('topcom_points2allfinetriangs', True)
    """
    def __init__(self, name):
        r"""
        TESTS::

            sage: from sage.features.topcom import TOPCOMExecutable
            sage: isinstance(TOPCOMExecutable('points2finetriangs'), TOPCOMExecutable)
            True
        """
        Executable.__init__(self, name=f"topcom_{name}",
                            executable=name,
                            spkg="topcom")


class TOPCOM(JoinFeature):
    r"""
    A :class:`~sage.features.Feature` describing the presence of the executables
    which comes as a part of :ref:`TOPCOM <spkg_topcom>`.

    EXAMPLES::

        sage: from sage.features.topcom import TOPCOM
        sage: TOPCOM().is_present()                             # optional - topcom
        FeatureTestResult('topcom', True)
    """
    def __init__(self):
        r"""
        TESTS::

            sage: from sage.features.topcom import TOPCOM
            sage: isinstance(TOPCOM(), TOPCOM)
            True
        """
        JoinFeature.__init__(self, "topcom",
                             [TOPCOMExecutable(name)
                              for name in ('points2allfinetriangs',)])


def all_features():
    return [TOPCOM()]



# Sage version information for Python scripts
# This file is auto-generated by the sage-update-version script, do not edit!
version = '10.5.beta0'
date = '2024-07-24'
banner = 'SageMath version 10.5.beta0, Release Date: 2024-07-24'



# sage_setup: distribution = sagemath-environment
r"""
Check for poppler features

poppler-utils is a collection of tools built on Poppler's library API, to
manage PDF and extract contents:

 - ``pdfattach`` -- add a new embedded file (attachment) to an existing PDF
 - ``pdfdetach`` -- extract embedded documents from a PDF
 - ``pdffonts`` -- lists the fonts used in a PDF
 - ``pdfimages`` -- extract all embedded images at native resolution from a PDF
 - ``pdfinfo`` -- list all information of a PDF
 - ``pdfseparate`` -- extract single pages from a PDF
 - ``pdftocairo`` -- convert single pages from a PDF to vector or bitmap formats using cairo
 - ``pdftohtml`` -- convert PDF to HTML format retaining formatting
 - ``pdftoppm`` -- convert a PDF page to a bitmap
 - ``pdftops`` -- convert PDF to printable PS format
 - ``pdftotext`` -- extract all text from PDF
 - ``pdfunite`` -- merges several PDF

Currently we only check for the presence of ``pdftocairo``.
"""
# ****************************************************************************
#       Copyright (C) 2021 Sebastien Labbe <slabqc@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#                  https://www.gnu.org/licenses/
# ****************************************************************************

from . import Executable

class pdftocairo(Executable):
    r"""
    A :class:`sage.features.Feature` describing the presence of
    ``pdftocairo``

    EXAMPLES::

        sage: from sage.features.poppler import pdftocairo
        sage: pdftocairo().is_present()             # optional: pdftocairo
        FeatureTestResult('pdftocairo', True)
    """
    def __init__(self):
        r"""
        TESTS::

            sage: from sage.features.poppler import pdftocairo
            sage: isinstance(pdftocairo(), pdftocairo)
            True
        """
        Executable.__init__(self, "pdftocairo", executable="pdftocairo",
                            url="https://poppler.freedesktop.org/")

def all_features():
    return [pdftocairo()]



# sage_setup: distribution = sagemath-environment
r"""
Features for testing the presence of ``latte_int``
"""

# ****************************************************************************
#       Copyright (C) 2018 Vincent Delecroix
#                     2019 Frédéric Chapoton
#                     2021 Matthias Koeppe
#                     2021 Kwankyu Lee
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#                  https://www.gnu.org/licenses/
# ****************************************************************************

from . import Executable
from .join_feature import JoinFeature


LATTE_URL = "https://www.math.ucdavis.edu/~latte/software.php"


class Latte_count(Executable):
    r"""
    Feature for the executable ``count`` from :ref:`LattE integrale <spkg_latte_int>`.
    """
    def __init__(self):
        r"""
        TESTS::

            sage: from sage.features.latte import Latte_count
            sage: isinstance(Latte_count(), Latte_count)
            True
        """
        Executable.__init__(self, "count", executable="count",
                            spkg="latte_int",
                            url=LATTE_URL)


class Latte_integrate(Executable):
    r"""
    Feature for the executable ``integrate`` from :ref:`LattE integrale <spkg_latte_int>`.
    """
    def __init__(self):
        r"""
        TESTS::

            sage: from sage.features.latte import Latte_integrate
            sage: isinstance(Latte_integrate(), Latte_integrate)
            True
        """
        Executable.__init__(self, "integrate", executable="integrate",
                            spkg="latte_int",
                            url=LATTE_URL)


class Latte(JoinFeature):
    r"""
    A :class:`~sage.features.Feature` describing the presence of excecutables
    from :ref:`LattE integrale <spkg_latte_int>`.

    EXAMPLES::

        sage: from sage.features.latte import Latte
        sage: Latte().is_present()  # optional - latte_int
        FeatureTestResult('latte_int', True)
    """
    def __init__(self):
        r"""
        TESTS::

            sage: from sage.features.latte import Latte
            sage: isinstance(Latte(), Latte)
            True
        """
        JoinFeature.__init__(self, "latte_int",
                             (Latte_count(), Latte_integrate()),
                             description="LattE")


def all_features():
    return [Latte()]



# sage_setup: distribution = sagemath-environment
r"""
Features for testing the presence of various databases
"""

# *****************************************************************************
#       Copyright (C) 2016      Julian Rüth
#                     2018-2019 Jeroen Demeyer
#                     2018      Timo Kaufmann
#                     2020-2022 Matthias Koeppe
#                     2020-2022 Sebastian Oehms
#                     2021      Kwankyu Lee
#
#  Distributed under the terms of the GNU General Public License (GPL)
#  as published by the Free Software Foundation; either version 2 of
#  the License, or (at your option) any later version.
#                  https://www.gnu.org/licenses/
# *****************************************************************************

import os

from . import StaticFile, PythonModule
from sage.env import SAGE_DATA_PATH


def sage_data_path(data_name):
    r"""
    Search path for database `data_name`.

    EXAMPLES::

        sage: from sage.features.databases import sage_data_path
        sage: sage_data_path("cremona")
        ['.../cremona']
    """
    if not SAGE_DATA_PATH:
        return []

    return [os.path.join(p, data_name)
            for p in SAGE_DATA_PATH.split(os.pathsep)]


class DatabaseCremona(StaticFile):
    r"""
    A :class:`~sage.features.Feature` which describes the presence of :ref:`John Cremona's
    database of elliptic curves <spkg_database_cremona_ellcurve>`.

    INPUT:

    - ``name`` -- either ``'cremona'`` (the default) for the full large
      database or ``'cremona_mini'`` for the small database.

    EXAMPLES::

        sage: from sage.features.databases import DatabaseCremona
        sage: DatabaseCremona('cremona_mini', type='standard').is_present()
        FeatureTestResult('database_cremona_mini_ellcurve', True)
        sage: DatabaseCremona().is_present()                                    # optional - database_cremona_ellcurve
        FeatureTestResult('database_cremona_ellcurve', True)
    """
    def __init__(self, name="cremona", spkg="database_cremona_ellcurve", type='optional'):
        r"""
        TESTS::

            sage: from sage.features.databases import DatabaseCremona
            sage: isinstance(DatabaseCremona(), DatabaseCremona)
            True
        """
        from sage.env import CREMONA_MINI_DATA_DIR, CREMONA_LARGE_DATA_DIR
        CREMONA_DATA_DIRS = set([CREMONA_MINI_DATA_DIR, CREMONA_LARGE_DATA_DIR])
        CREMONA_DATA_DIRS.discard(None)
        search_path = CREMONA_DATA_DIRS or sage_data_path("cremona")

        spkg = "database_cremona_ellcurve"
        spkg_type = "optional"
        if name == 'cremona_mini':
            spkg = "elliptic_curves"
            spkg_type = "standard"

        StaticFile.__init__(self, f"database_{name}_ellcurve",
                            filename=f"{name}.db",
                            search_path=search_path,
                            spkg=spkg,
                            type=spkg_type,
                            url="https://github.com/JohnCremona/ecdata",
                            description="Cremona's database of elliptic curves")


class DatabaseEllcurves(StaticFile):
    r"""
    A :class:`~sage.features.Feature` which describes the presence of
    William Stein's database of interesting curves.

    EXAMPLES::

        sage: from sage.features.databases import DatabaseEllcurves
        sage: bool(DatabaseEllcurves().is_present())  # optional - database_ellcurves
        True
    """
    def __init__(self):
        r"""
        TESTS::

            sage: from sage.features.databases import DatabaseEllcurves
            sage: isinstance(DatabaseEllcurves(), DatabaseEllcurves)
            True
        """
        from sage.env import ELLCURVE_DATA_DIR
        search_path = ELLCURVE_DATA_DIR or sage_data_path("ellcurves")

        StaticFile.__init__(self, "database_ellcurves",
                            filename='rank0',
                            search_path=search_path,
                            spkg="elliptic_curves",
                            type="standard",
                            description="William Stein's database of interesting curve")


class DatabaseGraphs(StaticFile):
    r"""
    A :class:`~sage.features.Feature` which describes the presence of
    the graphs database.

    EXAMPLES::

        sage: from sage.features.databases import DatabaseGraphs
        sage: bool(DatabaseGraphs().is_present())  # optional - database_graphs
        True
    """
    def __init__(self):
        r"""
        TESTS::

            sage: from sage.features.databases import DatabaseGraphs
            sage: isinstance(DatabaseGraphs(), DatabaseGraphs)
            True
        """
        from sage.env import GRAPHS_DATA_DIR
        search_path = GRAPHS_DATA_DIR or sage_data_path("graphs")

        StaticFile.__init__(self, "database_graphs",
                            filename='graphs.db',
                            search_path=search_path,
                            spkg="graphs",
                            type="standard",
                            description="A database of graphs")


class DatabaseJones(StaticFile):
    r"""
    A :class:`~sage.features.Feature` which describes the presence of
    :ref:`John Jones's tables of number fields <spkg_database_jones_numfield>`.

    EXAMPLES::

        sage: from sage.features.databases import DatabaseJones
        sage: bool(DatabaseJones().is_present())  # optional - database_jones_numfield
        True
    """
    def __init__(self):
        r"""
        TESTS::

            sage: from sage.features.databases import DatabaseJones
            sage: isinstance(DatabaseJones(), DatabaseJones)
            True
        """
        StaticFile.__init__(self, "database_jones_numfield",
                            filename='jones.sobj',
                            search_path=sage_data_path("jones"),
                            spkg="database_jones_numfield",
                            description="John Jones's tables of number fields")


class DatabaseKnotInfo(PythonModule):
    r"""
    A :class:`~sage.features.Feature` which describes the presence of the
    :ref:`package providing the KnotInfo and LinkInfo databases <spkg_database_knotinfo>`.

    The homes of these databases are the
    web-pages `KnotInfo <https://knotinfo.math.indiana.edu/>`__ and
    `LinkInfo <https://linkinfo.sitehost.iu.edu>`__.

    EXAMPLES::

        sage: from sage.features.databases import DatabaseKnotInfo
        sage: DatabaseKnotInfo().is_present()  # optional - database_knotinfo
        FeatureTestResult('database_knotinfo', True)
    """
    def __init__(self):
        r"""
        TESTS::

            sage: from sage.features.databases import DatabaseKnotInfo
            sage: isinstance(DatabaseKnotInfo(), DatabaseKnotInfo)
            True
        """
        PythonModule.__init__(self, 'database_knotinfo', spkg='database_knotinfo')


class DatabaseMatroids(PythonModule):
    r"""
    A :class:`~sage.features.Feature` which describes the presence of
    :ref:`Yoshitake Matsumoto's Database of Matroids <spkg_matroid_database>`.

    EXAMPLES::

        sage: from sage.features.databases import DatabaseMatroids
        sage: DatabaseMatroids().is_present()                                           # optional - matroid_database
        FeatureTestResult('matroid_database', True)

    REFERENCES:

    [Mat2012]_
    """
    def __init__(self):
        r"""
        TESTS::

            sage: from sage.features.databases import DatabaseMatroids
            sage: isinstance(DatabaseMatroids(), DatabaseMatroids)
            True
        """
        PythonModule.__init__(self, "matroid_database", spkg="matroid_database")


class DatabaseCubicHecke(PythonModule):
    r"""
    A :class:`~sage.features.Feature` which describes the presence of the
    :ref:`Cubic Hecke algebra database package <spkg_database_cubic_hecke>`.

    The home of this database is the
    web-page `Cubic Hecke algebra on 4 strands <http://www.lamfa.u-picardie.fr/marin/representationH4-en.html>`__
    of Ivan Marin.

    EXAMPLES::

        sage: from sage.features.databases import DatabaseCubicHecke
        sage: DatabaseCubicHecke().is_present()  # optional - database_cubic_hecke
        FeatureTestResult('database_cubic_hecke', True)
    """
    def __init__(self):
        r"""
        TESTS::

            sage: from sage.features.databases import DatabaseCubicHecke
            sage: isinstance(DatabaseCubicHecke(), DatabaseCubicHecke)
            True
        """
        PythonModule.__init__(self, 'database_cubic_hecke', spkg='database_cubic_hecke')


class DatabaseReflexivePolytopes(StaticFile):
    r"""
    A :class:`~sage.features.Feature` which describes the presence of the
    :ref:`PALP databases of reflexive three-dimensional <spkg_polytopes_db>`
    and :ref:`four-dimensional lattice polytopes <spkg_polytopes_db_4d>`.

    EXAMPLES::

        sage: from sage.features.databases import DatabaseReflexivePolytopes
        sage: bool(DatabaseReflexivePolytopes().is_present())                   # optional - polytopes_db
        True
        sage: bool(DatabaseReflexivePolytopes('polytopes_db_4d').is_present())  # optional - polytopes_db_4d
        True
    """
    def __init__(self, name='polytopes_db'):
        """
        TESTS::

            sage: from sage.features.databases import DatabaseReflexivePolytopes
            sage: isinstance(DatabaseReflexivePolytopes(), DatabaseReflexivePolytopes)
            True
            sage: DatabaseReflexivePolytopes().filename
            'Full3d'
            sage: DatabaseReflexivePolytopes('polytopes_db_4d').filename
            'Hodge4d'
        """
        from sage.env import POLYTOPE_DATA_DIR
        search_path = POLYTOPE_DATA_DIR or sage_data_path("reflexive_polytopes")

        dirname = "Full3d"
        if name == "polytopes_db_4d":
            dirname = "Hodge4d"

        StaticFile.__init__(self, name,
                            filename=dirname,
                            search_path=search_path)


def all_features():
    return [PythonModule('conway_polynomials', spkg='conway_polynomials', type='standard'),
            DatabaseCremona(),
            DatabaseCremona('cremona_mini', type='standard'),
            DatabaseEllcurves(),
            DatabaseGraphs(),
            DatabaseJones(),
            DatabaseKnotInfo(),
            DatabaseMatroids(),
            DatabaseCubicHecke(),
            DatabaseReflexivePolytopes(),
            DatabaseReflexivePolytopes('polytopes_db_4d')]
